library TestLib;

uses
  Windows,
  Messages,
  TestDef in 'TestDef.pas';

{$E dll}

{------------------------------------------------------------------------------}
{ Communication - project dependent code }

const
  CommWndClassName = TestLibCommWndClassName;
  CommCreatedName  = TestLibCommCreatedName;
  CommDestroyName  = TestLibCommDestroyName;
  CommRequestName  = TestLibCommRequestName;
  CommCommandName  = TestLibCommCommandName;

function CommWndOnCommand(WParam: WPARAM; LParam: LPARAM): LRESULT;
const
  Format = '[OnCommand]'#13#10'WParam: %8.8X'#13#10'LParam: %8.8X';
var
  ArgList: array [0..1] of LongInt;
  Text: array [0..1024] of Char;
begin
  Result := 0;
  ArgList[0] := WParam;
  ArgList[1] := LParam;
  wvsprintf(Text, Format, PChar(@ArgList));
  MessageBox(0, Text, 'Test_Lib', 0);
end;

function CommWndOnCommandData(Sender: HWND; Data: PCopyDataStruct): BOOL;
var
  Text: array [0..1024] of Char;
begin
  Result := True;
  lstrcpy(Text, '[OnCommandData]'#13#10);
  lstrcpyn(@Text[lstrlen(Text)], Data.lpData,
    SizeOf(Text) - lstrlen(Text));
  MessageBox(0, Text, 'Test_Lib', 0);
end;

{------------------------------------------------------------------------------}
{ Communication interface for libraries through an window                      }
{ - DllMain creates an thread when loading into an process                     }
{ - Thread registers an window class and creates an window                     }
{ - Window registers projekt dependent messages                                }
{ - Window braodcasts an CommCreatedMsg message when created (for monitoring)  }
{ - Window braodcasts an CommDestroyMsg message when destroying (monitoring)   }
{ - Window replys to an (broadcast) request (receiver/hWnd in WParam)          }
{ - Window calls CommWndOnCommand on CommDestroyMsg message                    }
{ - Window calls CommWndOnCommandData on WM_COPYDATA message (validated)       }
{ - DllMain closes the window and terminates the thread on unloading           }
{------------------------------------------------------------------------------}

{------------------------------------------------------------------------------}
{ Communication window }

var
  CommCreatedMsg: UINT;
  CommDestroyMsg: UINT;
  CommRequestMsg: UINT;
  CommCommandMsg: UINT;

function CommWndOnCreate(hWnd: HWND): LRESULT;
begin
  Result := 0;
  CommCreatedMsg := RegisterWindowMessage(CommCreatedName);
  CommDestroyMsg := RegisterWindowMessage(CommDestroyName);
  CommRequestMsg := RegisterWindowMessage(CommRequestName);
  CommCommandMsg := RegisterWindowMessage(CommCommandName);
  SendMessage(HWND_BROADCAST, CommCreatedMsg, WPARAM(hWnd), 0);
end;

function CommWndOnDestroy(hWnd: HWND): LRESULT;
begin
  Result := 0;
  PostQuitMessage(0);
  SendMessage(HWND_BROADCAST, CommDestroyMsg, WPARAM(hWnd), 0);
end;

function CommWndProc(hWnd: HWND; uMsg: UINT; WParam: WPARAM; LParam: LPARAM): LRESULT; stdcall;
begin
  case uMsg of
    WM_CREATE:
      Result := CommWndOnCreate(hWnd);
    WM_DESTROY:
      Result := CommWndOnDestroy(hWnd);
    WM_COPYDATA:
      if PCopyDataStruct(LParam)^.dwData = CommCommandMsg then
        Result := LRESULT(CommWndOnCommandData(DWORD(WParam), PCopyDataStruct(LParam)))
      else
        Result := DefWindowProc(hWnd, uMsg, WParam, LParam);
  else
    if (uMsg = CommRequestMsg) and (CommRequestMsg <> 0) and
      IsWindow(LongWord(WParam)) then
    begin
      Result := 0;
      SendMessage(LongWord(WParam), uMsg, LongInt(hWnd), 0);
    end
    else if (uMsg = CommCommandMsg) and (CommCommandMsg <> 0) then
      Result := CommWndOnCommand(WParam, LParam)
    else
      Result := DefWindowProc(hWnd, uMsg, WParam, LParam);
  end;
end;

var
  CommThreadWndClass: TWndClassEx = (cbSize: SizeOf(TWndClassEx); style: 0;
    lpfnWndProc: @CommWndProc; cbClsExtra: 0; cbWndExtra: 0; hInstance: 0;
    hIcon: 0; hCursor: 0; hbrBackground: 0; lpszMenuName : nil;
    lpszClassName: CommWndClassName; hIconSm: 0);
  CommThreadCreateStruct: TCreateStruct = (lpCreateParams: nil; hInstance: 0;
    hMenu: 0; hwndParent: 0; cy: 1; cx: 1; y: 0; x: 0; style: WS_OVERLAPPED;
    lpszName: nil; lpszClass: CommWndClassName; dwExStyle: WS_EX_TOOLWINDOW);
  CommThreadWnd: HWND;

{------------------------------------------------------------------------------}
{ Communication thread }

function CommThreadFunc(Param: Pointer): DWORD; stdcall;
var
  Msg: TMsg;
begin
  Msg.lParam := ERROR_INTERNAL_ERROR;
  CommThreadWndClass.hInstance := HInstance;
  if RegisterClassEx(CommThreadWndClass) <> 0 then
  begin
    CommThreadCreateStruct.hInstance := CommThreadWndClass.hInstance;
    with CommThreadCreateStruct do
      CommThreadWnd := CreateWindowEx(dwExStyle, lpszClass, lpszName, style,
        x, y, cx, cy, hwndParent, hMenu, hInstance, lpCreateParams);
    if CommThreadWnd <> 0 then
      while GetMessage(Msg, 0, 0, 0) do
      begin
        TranslateMessage(Msg);
        DispatchMessage(Msg);
      end;
    with CommThreadWndClass do
      UnregisterClass(lpszClassName, hInstance);
  end;
  Result := DWORD(Msg.lParam);
end;

{------------------------------------------------------------------------------}
{ DllMain }

var
  CommThread: THandle;
  CommThrdId: DWORD;

procedure LibraryProc(Reason: Integer);
var
  Code: DWORD;
begin
  case Reason of
    DLL_PROCESS_DETACH:
      begin
        if CommThreadWnd <> 0 then
          SendMessage(CommThreadWnd, WM_CLOSE, 0, 0);
        if CommThread <> 0 then
        begin
          Code := STILL_ACTIVE;
          GetExitCodeThread(CommThread, Code);
          if Code = STILL_ACTIVE then
          begin
            WaitForSingleObject(CommThread, 500);
            TerminateThread(CommThread, 0);
          end;
          CloseHandle(CommThread);
          CommThread := 0;
          CommThrdId := 0;
        end;
      end;
    DLL_PROCESS_ATTACH:
      begin
        DisableThreadLibraryCalls(HInstance);  // For optimization purposes only
        CommThread := CreateThread(nil, 0, @CommThreadFunc, nil, 0, CommThrdId);
      end;
  end;
end;

begin
  DllProc := @LibraryProc;
  LibraryProc(DLL_PROCESS_ATTACH);
end.
