unit TestMonMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, TestDef;

type
  TMainForm = class(TForm)
    RichEdit1: TRichEdit;
    BtnClear: TButton;
    BtnFind: TButton;
    BtnRequest: TButton;
    EdCommand: TEdit;
    BtnCommand: TButton;
    BtnPost: TButton;
    EdWParam: TEdit;
    EdLParam: TEdit;
    EdData: TEdit;
    BtnData: TButton;
    procedure BtnClearClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BtnFindClick(Sender: TObject);
    procedure BtnRequestClick(Sender: TObject);
    procedure BtnCommandClick(Sender: TObject);
    procedure BtnPostClick(Sender: TObject);
    procedure BtnDataClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure DefaultHandler(var Message); override;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.BtnClearClick(Sender: TObject);
begin
  RichEdit1.Clear;
end;

var
  TestLibCommCreatedMsg: UINT;
  TestLibCommDestroyMsg: UINT;
  TestLibCommRequestMsg: UINT;
  TestLibCommCommandMsg: UINT;

{------------------------------------------------------------------------------}
{ Initialize the message IDs }

procedure TMainForm.FormCreate(Sender: TObject);
begin
  TestLibCommCreatedMsg := RegisterWindowMessage(TestLibCommCreatedName);
  TestLibCommDestroyMsg := RegisterWindowMessage(TestLibCommDestroyName);
  TestLibCommRequestMsg := RegisterWindowMessage(TestLibCommRequestName);
  TestLibCommCommandMsg := RegisterWindowMessage(TestLibCommCommandName);
end;

{------------------------------------------------------------------------------}
{ Capture all interesting messages }

procedure TMainForm.DefaultHandler(var Message);
begin
  if Assigned(RichEdit1) then
  begin
    with TMessage(Message), RichEdit1.Lines do
      if (Msg = TestLibCommCreatedMsg) and (TestLibCommCreatedMsg <> 0) then
        Add('Test Created - CommWnd: $' + IntToHex(HWND(WParam), 8))
      else if (Msg = TestLibCommDestroyMsg) and (TestLibCommDestroyMsg <> 0) then
        Add('Test Destroy - CommWnd: $' + IntToHex(HWND(WParam), 8))
      else if (Msg = TestLibCommRequestMsg) and (TestLibCommRequestMsg <> 0) and
        (HWND(WParam) <> Handle) then
        Add('Test Request - CommWnd: $' + IntToHex(HWND(WParam), 8))
      else
        inherited;
  end
  else
    inherited;
end;

{------------------------------------------------------------------------------}
{ Look for communication windows by class name }

function EnumWindowsProc(hWnd: HWND; LParam: LPARAM): BOOL; stdcall;
var
  Buffer: array [0..1023]of Char;
begin
  Result := True;
  if GetClassName(hWnd, Buffer, SizeOf(Buffer)) > 0 then
  begin
    if StrComp(Buffer, TestLibCommWndClassName) = 0 then
      TStrings(LParam).Add('Found - CommWnd: $' + IntToHex(LongWord(hWnd), 8))
  end;
end;

procedure TMainForm.BtnFindClick(Sender: TObject);
begin
  EnumWindows(@EnumWindowsProc, LPARAM(RichEdit1.Lines));
end;

{------------------------------------------------------------------------------}
{ Send request to all windows--answer is handled in DefaultHandler }

procedure TMainForm.BtnRequestClick(Sender: TObject);
begin
  SendMessage(HWND_BROADCAST, TestLibCommRequestMsg, WPARAM(Handle), 0);
end;

{------------------------------------------------------------------------------}
{ Send command message to specific communication window - synchronous }

procedure TMainForm.BtnCommandClick(Sender: TObject);
var
  Wnd: HWND;
  WParam: LongInt;
  LParam: LongInt;
begin
  Wnd := StrToIntDef('$' + EdCommand.Text, 0);
  WParam := StrToIntDef('$' + EdWParam.Text, 0);
  LParam := StrToIntDef('$' + EdLParam.Text, 0);
  if (Wnd <> 0) and IsWindow(Wnd) then
    SendMessage(Wnd, TestLibCommCommandMsg, WParam, LParam)
  else
    ShowMessage('Invalid window handle specified');
end;

{------------------------------------------------------------------------------}
{ Post command message to specific communication window - asynchronous }

procedure TMainForm.BtnPostClick(Sender: TObject);
var
  Wnd: HWND;
  WParam: LongInt;
  LParam: LongInt;
begin
  Wnd := StrToIntDef('$' + EdCommand.Text, 0);
  WParam := StrToIntDef('$' + EdWParam.Text, 0);
  LParam := StrToIntDef('$' + EdLParam.Text, 0);
  if (Wnd <> 0) and IsWindow(Wnd) then
    PostMessage(Wnd, TestLibCommCommandMsg, WParam, LParam)
  else
    ShowMessage('Invalid window handle specified');
end;

{------------------------------------------------------------------------------}
{ Send command data message to specific communication window - synchronous }

procedure TMainForm.BtnDataClick(Sender: TObject);
var
  Wnd: HWND;
  Buffer: array [0..1024] of Char;
  Data: TCopyDataStruct;
begin
  Wnd := StrToIntDef('$' + EdCommand.Text, 0);
  if (Wnd <> 0) and IsWindow(Wnd) then
  begin
    FillChar(Buffer, SizeOf(Buffer), 0);
    StrPLCopy(Buffer, EdData.Text, SizeOf(Buffer) - 1);
    Data.dwData := TestLibCommCommandMsg;
    Data.cbData := SizeOf(Buffer);
    Data.lpData := @Buffer[0];
    SendMessage(Wnd, WM_COPYDATA, WPARAM(Handle), LPARAM(@Data));
  end
  else
    ShowMessage('Invalid window handle specified');
end;

end.
